from ..base import BuildStage
from pathlib import Path
from prototyper.conf import DJANGO_TARGET
from django.core.management.utils import get_random_secret_key


TPL = """\"\"\"
Django settings for {project_name} project.

Generated by django-prototyper
https://github.com/vitalik/django-prototyper

For more information on this file, see
https://docs.djangoproject.com/en/{django_version}/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/{django_version}/ref/settings/
\"\"\"

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = {BASE_DIR}


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/{django_version}/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '{SECRET_KEY}'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    {INSTALLED_APPS}
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = '{url_conf}'

TEMPLATES = [
    {{
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {{
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        }},
    }},
]

WSGI_APPLICATION = '{wsgi_app}'


# Database
# https://docs.djangoproject.com/en/{django_version}/ref/settings/#databases

DATABASES = {{
    'default': {{
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }}
}}


# Password validation
# https://docs.djangoproject.com/en/{django_version}/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {{
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    }},
    {{
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    }},
    {{
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    }},
    {{
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    }},
]


# Internationalization
# https://docs.djangoproject.com/en/{django_version}/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/{django_version}/howto/static-files/

STATIC_URL = '/static/'
"""


class SettingsStage(BuildStage):
    def run(self):
        root = Path(self.build.settings_pckg_path)
        if not root.exists():
            root.mkdir()
        (root / '__init__.py').touch()
        settings_py = root / 'settings.py'
        content = self._get_content()
        settings_py.write_text(content)
    
    def _get_content(self):
        proj_name = self.build.project.name
        BASE_DIR = 'os.path.dirname(os.path.abspath(__file__))'
        if self.build.is_settings_py_separate():
            BASE_DIR = 'os.path.dirname({0})'.format(BASE_DIR)
        ctx = {
            'project_name': proj_name,
            'SECRET_KEY': get_random_secret_key(),
            'url_conf': self.settings_module('urls'),
            'wsgi_app': self.settings_module('wsgi.application'),
            'INSTALLED_APPS': self._installed_apps_lines(),
            'django_version': '{v[0]}.{v[1]}'.format(v=DJANGO_TARGET),
            'BASE_DIR': BASE_DIR,
        }
        result = TPL.format(**ctx)
        return result
    
    def _installed_apps_lines(self):
        result = [
            "'django.contrib.admin',",
            "'django.contrib.auth',",
            "'django.contrib.contenttypes',",
            "'django.contrib.sessions',",
            "'django.contrib.messages',",
            "'django.contrib.staticfiles',",
            "",
        ]

        for plugin in self.build.details['plugins']:
            apps = plugin.get('apps', [])
            for app in apps:
                result.append("'{0}',".format(app['name']))
        
        result.append('')

        for app in self.build.details['apps']:
            if not app['external']:
                result.append("'{0}',".format(app['name']))
        return '\n    '.join(result)
